@@ -1,23 +1,38 @@
 /*
-** S390 specific part of trace.c
-**
-** Other routines are in ../trace.c and need to be combined
-** at link time with this code.
-**
-** Copyright (C) 2001,2005 IBM Corp.
-*/
+ * This file is part of ltrace.
+ * Copyright (C) 2012 Petr Machata, Red Hat Inc.
+ * Copyright (C) 2001,2005 IBM Corp.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301 USA
+ */
 
 #include "config.h"
 
-#include <errno.h>
-#include <stdlib.h>
+#include <asm/ptrace.h>
+#include <sys/ptrace.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <assert.h>
+#include <errno.h>
 #include <signal.h>
-#include <sys/ptrace.h>
-#include <asm/ptrace.h>
+#include <stdlib.h>
 
 #include "common.h"
+#include "proc.h"
+#include "type.h"
 
 #if (!defined(PTRACE_PEEKUSER) && defined(PTRACE_PEEKUSR))
 # define PTRACE_PEEKUSER PTRACE_PEEKUSR
@@ -159,40 +174,84 @@ syscall_p(Process *proc, int status, int *sysnum) {
 	return 0;
 }
 
-long
-gimme_arg(enum tof type, Process *proc, int arg_num, arg_type_info *info) {
-	long ret;
+size_t
+arch_type_sizeof(struct Process *proc, struct arg_type_info *info)
+{
+	if (proc == NULL)
+		return (size_t)-2;
+
+	switch (info->type) {
+	case ARGTYPE_VOID:
+		return 0;
+
+	case ARGTYPE_CHAR:
+		return 1;
+
+	case ARGTYPE_SHORT:
+	case ARGTYPE_USHORT:
+		return 2;
+
+	case ARGTYPE_INT:
+	case ARGTYPE_UINT:
+		return 4;
+
+	case ARGTYPE_LONG:
+	case ARGTYPE_ULONG:
+	case ARGTYPE_POINTER:
+		return proc->e_class == ELFCLASS64 ? 8 : 4;
+
+	case ARGTYPE_FLOAT:
+		return 4;
+	case ARGTYPE_DOUBLE:
+		return 8;
+
+	case ARGTYPE_ARRAY:
+	case ARGTYPE_STRUCT:
+		/* Use default value.  */
+		return (size_t)-2;
 
-	switch (arg_num) {
-	case -1:		/* return value */
-		ret = ptrace(PTRACE_PEEKUSER, proc->pid, PT_GPR2, 0);
-		break;
-	case 0:
-		ret = ptrace(PTRACE_PEEKUSER, proc->pid, PT_ORIGGPR2, 0);
-		break;
-	case 1:
-		ret = ptrace(PTRACE_PEEKUSER, proc->pid, PT_GPR3, 0);
-		break;
-	case 2:
-		ret = ptrace(PTRACE_PEEKUSER, proc->pid, PT_GPR4, 0);
-		break;
-	case 3:
-		ret = ptrace(PTRACE_PEEKUSER, proc->pid, PT_GPR5, 0);
-		break;
-	case 4:
-		ret = ptrace(PTRACE_PEEKUSER, proc->pid, PT_GPR6, 0);
-		break;
 	default:
-		fprintf(stderr, "gimme_arg called with wrong arguments\n");
-		exit(2);
+		assert(info->type != info->type);
+		abort();
 	}
-#ifdef __s390x__
-	if (proc->mask_32bit)
-		ret &= 0xffffffff;
-#endif
-	return ret;
 }
 
-void
-save_register_args(enum tof type, Process *proc) {
+size_t
+arch_type_alignof(struct Process *proc, struct arg_type_info *info)
+{
+	if (proc == NULL)
+		return (size_t)-2;
+
+	switch (info->type) {
+	default:
+		assert(info->type != info->type);
+		break;
+
+	case ARGTYPE_CHAR:
+		return 1;
+
+	case ARGTYPE_SHORT:
+	case ARGTYPE_USHORT:
+		return 2;
+
+	case ARGTYPE_INT:
+	case ARGTYPE_UINT:
+		return 4;
+
+	case ARGTYPE_LONG:
+	case ARGTYPE_ULONG:
+	case ARGTYPE_POINTER:
+		return proc->e_class == ELFCLASS64 ? 8 : 4;
+
+	case ARGTYPE_FLOAT:
+		return 4;
+	case ARGTYPE_DOUBLE:
+		return 8;
+
+	case ARGTYPE_ARRAY:
+	case ARGTYPE_STRUCT:
+		/* Use default value.  */
+		return (size_t)-2;
+	}
+	abort();
 }
